/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//`define SUPPORT_PG		// ALU_WIDTH == 4 only
//`define SUPPORT_SIO_COMATIBLE

//
// Module Name: alu181
//   ver 001
// 
//         original  74181 
//    B0  1            24  VCC  
//    A0  2            23  A1 
//    S3  3            22  B1 
//    S2  4            21  A2  
//    S1  5            20  B2 
//    S0  6            19  A3  
//    Cn  7            18  B3  
//     M  8            17  ~G
//    F0  9            16  Cn+4
//    F1  10           15  ~P
//    F2  11           14  EQ 
//   GND  12           13  F3 
//
// XO2-256      modified  74181      XO2-256
//  27        B0  1            24  VCC  
//  28        A0  2            23  A1    25
//  29        S3  3            22  B1    23
//  30        S2  4            21  A2    21
//  32        S1  5            20  B2    20
//   1        S0  6            19  A3    17
//   4        Cn  7            18  B3    16
//   5         M  8          x 17  N.C.  26
//   8        F0  9            16  Cn+4  14
//   9        F1  10         x 15  N.C.  13
//  10        F2  11           14  EQ    12
//           GND  12           13  F3    11


module alu181 # (
	parameter ALU_WIDTH = 4
) (
		 input   C_IN       // Carry Input (INV)
		, input  [ALU_WIDTH-1:0] A     // DATA IN
		, input  [ALU_WIDTH-1:0] B     // DATA IN
		, output [ALU_WIDTH-1:0] F   // DATA OUT
		, output  EQ         // A == B
		, output  C_OUT      // carry-look-ahead (INV)
`ifdef SUPPORT_PG
		, output  P          // carry-propagate (INV)
		, output  G          // carry-generate (INV)
`endif
		, input   M          // Mode Control
		, input   [3:0] S   // Function Select
		);

    wire [3:0] D;
    wire [3:0] C;
    assign D[0] = ~( A[0] | B[0] & S[0] | ~B[0] & S[1] );
    assign D[1] = ~( A[1] | B[1] & S[0] | ~B[1] & S[1] );
    assign D[2] = ~( A[2] | B[2] & S[0] | ~B[2] & S[1] );
    assign D[3] = ~( A[3] | B[3] & S[0] | ~B[3] & S[1] );

    assign C[0] = ~( A[0] & ~B[0] & S[2] | A[0] & B[0] & S[3] );
    assign C[1] = ~( A[1] & ~B[1] & S[2] | A[1] & B[1] & S[3] );
    assign C[2] = ~( A[2] & ~B[2] & S[2] | A[2] & B[2] & S[3] );
    assign C[3] = ~( A[3] & ~B[3] & S[2] | A[3] & B[3] & S[3] );

    assign F[0] = ~( ~M & C_IN ) ^ D[0] ^ C[0];
    assign F[1] = ~( ~M & ( D[0] | C_IN & C[0] )) ^ D[1] ^ C[1];
    assign F[2] = ~( ~M & ( D[1] | D[0] & C[1] | C_IN & C[0] & C[1]))
			 ^ D[2] ^ C[2];
    assign F[3] = ~( ~M & ( D[2] | D[1] & C[2] | D[0] & C[1] & C[2] 
		     | C_IN & C[0] & C[1] & C[2])) ^ D[3] ^ C[3];

    wire Yinv = D[3] | C[3] & ( D[2] | C[2] & ( D[1] | C[1] & D[0]));
    wire C_OUT = Yinv | C[3] & C[2] & C[1] & C[0] & C_IN;
    wire EQ = F[0] & F[1] & F[2] & F[3];
`ifdef SUPPORT_PG
    wire X = ~( C[3] & C[2] & C[1] & C[0] );
    wire G = ~ Yinv;
    wire P = X;
`endif
endmodule
