/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//`define SUPPORT_PG		// ALU_WIDTH == 4 only

//
// Module Name: alu181
//   ver 001
// 
//         original  74181 
//    B0  1            24  VCC  
//    A0  2            23  A1 
//    S3  3            22  B1 
//    S2  4            21  A2  
//    S1  5            20  B2 
//    S0  6            19  A3  
//    Cn  7            18  B3  
//     M  8            17  ~G
//    F0  9            16  Cn+4
//    F1  10           15  ~P
//    F2  11           14  EQ 
//   GND  12           13  F3 
//
// XO2-256      modified  74181      XO2-256
//  27        B0  1            24  VCC  
//  28        A0  2            23  A1    25
//  29        S3  3            22  B1    23
//  30        S2  4            21  A2    21
//  32        S1  5            20  B2    20
//   1        S0  6            19  A3    17
//   4        Cn  7            18  B3    16
//   5         M  8          x 17  N.C.  26
//   8        F0  9            16  Cn+4  14
//   9        F1  10         x 15  SWAP  13
//  10        F2  11           14  EQ    12
//           GND  12           13  F3    11


module alu181ex (
		 input   C_IN       // Carry Input (INV)
		, input   SWAP       // SWAP A/B
		, input  [3:0] A     // DATA IN
		, input  [3:0] B     // DATA IN
		, output [3:0] F   // DATA OUT
		, output  EQ         // A == B
		, output  C_OUT      // carry-look-ahead (INV)
`ifdef SUPPORT_PG
		, output  P          // carry-propagate (INV)
		, output  G          // carry-generate (INV)
`endif
		, input   M          // Mode Control
		, input   [3:0] S   // Function Select
		);

    wire cin = ~C_IN;
    wire cout;
    wire [7:0] FS;
    assign FS[0] = SWAP? 1'b1 : S[0];
    assign FS[1] = SWAP? 1'b0 : S[1];
    assign FS[2] = SWAP? 1'b0 : S[2];
    assign FS[3] = SWAP? 1'b1 : S[3];
    assign FS[4] = SWAP? S[0] : 1'b1;
    assign FS[5] = SWAP? S[1] : 1'b0;
    assign FS[6] = SWAP? S[2] : 1'b0;
    assign FS[7] = SWAP? S[3] : 1'b1;

	alu4 alu_impl ( .A(A[3:0]), .B(B[3:0]), .F(F[3:0]), .S(FS[7:0])
			, .M(M) , .C_IN(cin) , .C_OUT(cout)
`ifdef SUPPORT_PG
			,.P(P) , .G(G)
`endif
			);
	assign C_OUT = ~cout;
endmodule
