/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
module dac # (
		parameter WIDTH = 8
		, parameter WITH_LATCH = 1
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output A_OUT
	);

    reg r_stb = 1'b0;
    reg [WIDTH-1:0] r_data = 0;
    reg [WIDTH-1:0] count = 0;
    reg r_out = 1'b0;

    always @(posedge CLK)
    begin
	r_stb <= STB;
	if (&count | (STB & ~r_stb))
	  begin
		count <= 0;
		if (WITH_LATCH)
			r_data <= I_DATA;
		r_out <= 0;
	  end
	else
	  begin
		count <= count + 1;
		r_out <= ( count < (WITH_LATCH ? r_data : I_DATA) );
	  end
    end

    assign A_OUT = r_out;

endmodule
