/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Sigma Delta DAC using 3bit R-2R DAC */
module dac3 # (
		parameter WIDTH = 8
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output [2:0] A_OUT
	);

    reg r_stb = 1'b0;
    reg [WIDTH-1:0] r_data = 0;
    reg [WIDTH-3-1:0] fraction = 0;
    reg [WIDTH-3-1:0] count = 0;
    reg [2:0] r_out = 0;

    wire [WIDTH-3-1:0] WK = 0;
    wire [WIDTH-1:0] I_MAX = { 3'b110 , ~WK }; 
    wire [WIDTH-1:0] I_DATA2 = (&I_DATA[WIDTH-1:WIDTH-3])
		 		? I_DATA : I_MAX;

    always @(posedge CLK)
    begin
	r_stb <= STB;
	if (&count | (STB & ~r_stb))
	  begin
		count <= 0;
		r_data <= I_DATA2;
		fraction <= I_DATA2[WIDTH-3-1:0];
		r_out <= I_DATA2[WIDTH-1:WIDTH-3];
	  end
	else
	  begin
		{ r_out, fraction } <= { 3'b0 ,fraction } + r_data;
		count <= count + 1;
	  end
    end

    assign A_OUT = r_out;

endmodule
