/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Sigma Delta DAC using 3bit sub DAC (HI-Speed) */

module dacx # (
		parameter WIDTH = 8
	) (
		input CLK
		,input STB // active high
                ,input [WIDTH-1:0] I_DATA
		,output A_OUT
	);

	reg [2:0] hi_clk;
	wire [2:0] hi_out;
	wire lo_stb = CLK;

	always @(negedge CLK)
	begin
		hi_clk <= hi_clk + 1;
	end

	dac #( .WIDTH(3) , .WITH_LATCH(0) ) dac_lo ( .CLK(CLK), .STB(lo_stb)
		, .I_DATA(hi_out), .A_OUT(A_OUT) );
	dac5 #( .AWIDTH(3), .WIDTH(WIDTH-3) ) dac_hi ( .CLK(hi_clk[2]) , .STB(STB)
		, .I_DATA(I_DATA) , .A_OUT(hi_out) );

endmodule
