// Pseudo Random Sequence Generator
// http://www.xilinx.com/support/documentation/application_notes/xapp052.pdf
//
module lfsr #(
		parameter WIDTH=20
		//parameter WIDTH=24
		//parameter WIDTH=32
	) (
		inout CLK
		, output O
		);

    reg [1:0]  chk = 0;
    reg [WIDTH-1:0] r_lfsr = 1;
    reg r_out = 1'b0;

    assign O = r_out;

    always @(posedge CLK)
    begin
	chk <= { chk[0:0] , r_lfsr[0]};
	if ( { chk[1:0] , r_lfsr[0] } == 3'b000 )
		r_out <= 1'b1;
	else if ( { chk[1:0] , r_lfsr[0] } == 3'b111 )
		r_out <= 1'b0;
	else 
	  begin
		r_out <= r_lfsr[0];
		if (WIDTH == 16)
			r_lfsr <= { r_lfsr[0]
			   , ( r_lfsr[15] ^ r_lfsr[0]), r_lfsr[14:14]
			   , ( r_lfsr[13] ^ r_lfsr[0]), r_lfsr[12:5]
			   , ( r_lfsr[ 4] ^ r_lfsr[0]), r_lfsr[3:1] };
		else if (WIDTH == 20)
			r_lfsr <= { r_lfsr[0] , r_lfsr[19:18]
			   , ( r_lfsr[17] ^ r_lfsr[0]) , r_lfsr[16:1] };
		else if (WIDTH == 24)
			r_lfsr <= { r_lfsr[0]
			   , ( r_lfsr[23] ^ r_lfsr[0])
			   , ( r_lfsr[22] ^ r_lfsr[0]) , r_lfsr[21:18]
			   , ( r_lfsr[17] ^ r_lfsr[0]) , r_lfsr[16:1] };
		else if (WIDTH == 32)
			r_lfsr <= { r_lfsr[0], r_lfsr[31:23]
			   , ( r_lfsr[22] ^ r_lfsr[0]), r_lfsr[21:3]
			   , ( r_lfsr[2] ^ r_lfsr[0]) 
			   , ( r_lfsr[1] ^ r_lfsr[0]) };
		else if (WIDTH == 48)
			r_lfsr <= { r_lfsr[0]
			   , ( r_lfsr[47] ^ r_lfsr[0]) , r_lfsr[46:22]
			   , ( r_lfsr[21] ^ r_lfsr[0])
			   , ( r_lfsr[20] ^ r_lfsr[0]) , r_lfsr[19:1] };
	  end
    end
endmodule
