/*
 * Copyright (C) 2011-2012 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
//
//
`define TC_SUPPORT_DOWN
`define TC_SUPPORT_ALARM

module time_counter # (
        	// parameter MAX = 60
		parameter MAX = 24
	) (
                input CLK
		, input I_UP
		, input I_UP_ALT
`ifdef TC_SUPPORT_DOWN
		, input I_DN
`endif
		, input [1:0] SEL  // select output
                , output [3:0] O
		, output O_UP
`ifdef TC_SUPPORT_ALARM
		, output MATCH
`endif
		);

    reg [3:0] lower = 0;
    reg [3:0] upper = 0;
`ifdef  TC_SUPPORT_ALARM
    reg [3:0] lower_a = 0;
    reg [3:0] upper_a = 0;

    assign O[3:0] = (SEL == 0) ? upper[3:0]
		   :(SEL == 1) ? lower[3:0]
		   :(SEL == 2) ? upper_a[3:0]
		   :             lower_a[3:0]
		   ;

    assign MATCH = ( upper == upper_a ) & ( lower == lower_a );
`else
    assign O[3:0] = SEL[0] ? lower[3:0] : upper[3:0];
`endif
    reg r_ovr;

    assign O_UP = r_ovr;

    always @(negedge CLK)
    begin
	if (I_UP | I_UP_ALT & ~SEL[1])
	    if ((upper >= (MAX-1)/10) & (lower >= (MAX-1)%10))
		begin
		    r_ovr <= 1'b1;
		    upper <= 0;
		    lower  <= 0;
		end
	    else if (lower >= 9)
		begin
		    r_ovr <= 1'b0;
		    upper <= upper + 1;
		    lower  <= 0;
		end
	    else
		begin
		    r_ovr <= 1'b0;
		    lower  <= lower + 1;
		end
`ifdef TC_SUPPORT_DOWN
	else if (I_DN & ~SEL[1])
	    if ((upper == 0) & (lower == 0))
		begin
		    r_ovr <= 1'b1;
		    upper <=  (MAX-1)/10;
		    lower  <= (MAX-1)%10;
		end
	    else if (lower == 0)
		begin
		    r_ovr <= 1'b0;
		    upper <= upper - 1;
		    lower  <= 9;
		end
	    else
		begin
		    r_ovr <= 1'b0;
		    lower  <= lower - 1;
		end
`endif
	else
	    r_ovr <= 1'b0;
    end
`ifdef  TC_SUPPORT_ALARM
    always @(negedge CLK)
    begin
	if (I_UP_ALT & SEL[1])
	    if ((upper_a >= (MAX-1)/10) & (lower_a >= (MAX-1)%10))
		begin
		    upper_a <= 0;
		    lower_a  <= 0;
		end
	    else if (lower_a >= 9)
		begin
		    upper_a <= upper_a + 1;
		    lower_a  <= 0;
		end
	    else
		begin
		    lower_a  <= lower_a + 1;
		end
`ifdef TC_SUPPORT_DOWN
	else if (I_DN & SEL[1])
	    if ((upper_a == 0) & (lower_a == 0))
		begin
		    upper_a <=  (MAX-1)/10;
		    lower_a  <= (MAX-1)%10;
		end
	    else if (lower_a == 0)
		begin
		    upper_a <= upper_a - 1;
		    lower_a  <= 9;
		end
	    else
		begin
		    lower_a  <= lower_a - 1;
		end
`endif
    end
`endif
endmodule
